#include "fraktal.h"
#include <math.h>

Fraktal::Fraktal()
{
	m_Aufloesung = 2;
	m_JuliaAufloesung = 2;
	m_Mittelpunkt.set(0, 0);

	m_Exponent		= 2;
	m_isMandel		= true;
	m_Iterationen	= 20;
	m_Zoomfaktor	= 3;

	m_Col_Upper.setFarbe(255, 255, 0);	//gelb
	m_Col_Lower.setFarbe(255, 0, 0);	//rot
	m_Col_Menge.setFarbe(0, 0, 0);		//schwarz
}

double Fraktal::Betrag(cmplx c)
{
	return sqrt(pow(c.Re,2) + pow(c.Im,2));
}


cmplx Fraktal::Multiplikation(cmplx c, cmplx d)
{//Multiplikation komplexer Zahlen (zum Quadrieren benoetigt)
	cmplx e;
		
	e.Re = c.Re * d.Re - c.Im * d.Im;
	e.Im = c.Re * d.Im + c.Im * d.Re;
	return e; // e = c * d
}

void Fraktal::CalcRelations()
{
	double Aufloesung;
	if(m_isMandel == true)
		Aufloesung = m_Aufloesung;
	else
		Aufloesung = m_JuliaAufloesung;

	m_Relationsfaktor = (Aufloesung / (YSIZE/2));

	//Der Relationsfaktor wird fuer die Umrechnung der Screen-
	//in Mandelbrotkoordinaten benoetigt
}

double Fraktal::ScreenToMB(double Mittelpunkt, int xyScreen, bool xORy)
{
	xyScreen -= ((YSIZE)/2);
	double ReturnVal = m_Relationsfaktor * xyScreen + Mittelpunkt;

	if((m_isMandel == false) && (xORy == Y))
		return -ReturnVal;
	
	return ReturnVal;
}

void Fraktal::CreatePic(HWND hDlg)
{
	cmplx	C;
		C.Re = m_Mittelpunkt.getRe();	//evtl. konstantes C fuer Juliamenge
		C.Im = m_Mittelpunkt.getIm();

	cmplx	Z;
	int		Fortschritt = 0;
	char	stringFort[8];	//Zeichenkette fuer die Anzeige des Fortschritts

	for(int x=0; x < XSIZE; x++)
	{
		//Berechnung und Anzeige des Fortschritts:
		Fortschritt = ((100 * x) / XSIZE);
		wsprintf(stringFort, "%i %%", Fortschritt);
		SetDlgItemText(hDlg, ID_FORTSCHRITT, stringFort);

		for(int y=0; y < YSIZE; y++)
		{
			if(isMandel() == true)
			{
				C.Re = ScreenToMB(m_Mittelpunkt.getRe(), x, X);
				C.Im = ScreenToMB(m_Mittelpunkt.getIm(), y, Y);
					    
				Z.Re = C.Re;
				Z.Im = C.Im;

				int i = Iterationsschleife(C, Z);
				SetColor(i, x, y);				
			}
			else
			{
				Z.Re = ScreenToMB(m_JuliaMittelpunkt.getRe(), x, X);
				Z.Im = ScreenToMB(m_JuliaMittelpunkt.getIm(), y, Y);

				int i = Iterationsschleife(C, Z);
				SetColor(i, x, y);
			}
		}
	}

	SetDlgItemText(hDlg, ID_FORTSCHRITT, "100 %");

	m_MC.CalcAnteil();	//Anteil der Menge am Gesamtbild berechnen
}

int Fraktal::Iterationsschleife(cmplx C, cmplx Z)
{
	cmplx Zq;

	for(int i=0; i <= getIterationen(); i++)
	{
		Zq.Re = Z.Re;
		Zq.Im = Z.Im;

		if(getExponent() > 1)
		{
			for(int e=1; e < getExponent(); e++)
				Zq = Multiplikation(Zq, Z);
		}

		Z.Re = Zq.Re + C.Re;
		Z.Im = Zq.Im + C.Im;

		if(Betrag(Z) > 2) break;
	}

    return i;
}

void Fraktal::SetColor(int i, int x, int y)
{
	if(i >= getIterationen())
	{
		m_Fraktalbild[x][y] = RGB(m_Col_Menge.getRed(),
								  m_Col_Menge.getGreen(),
								  m_Col_Menge.getBlue()
								 );
		m_MC.IncreaseTreffer();
	}
	else
	{
		int red, green, blue;
			
		int Differenz = m_Col_Upper.getRed() - m_Col_Lower.getRed();
		if(Differenz != 0)
			red = (abs(Differenz) / getIterationen()) * i;
		else
			red = m_Col_Upper.getRed();


		Differenz = m_Col_Upper.getGreen() - m_Col_Lower.getGreen();
		if(Differenz != 0)
			green = (abs(Differenz) / getIterationen()) * i;
		else
			green = m_Col_Upper.getGreen();
			Differenz = m_Col_Upper.getBlue() - m_Col_Lower.getBlue();
		if(Differenz != 0)
			blue = (abs(Differenz) / getIterationen()) * i;
		else
			blue = m_Col_Upper.getBlue();
			m_Fraktalbild[x][y] = RGB(red, green, blue);
	}
}