#include "main.h"

HWND		hWnd;
HDC			hdc;
HWND		hDlg;
PAINTSTRUCT	ps;
HINSTANCE	hInstance;
POINT		pt;

char		schrift[60];
Fraktal		Menge;

LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);
BOOL CALLBACK DialogProc(HWND, UINT, WPARAM, LPARAM);		//Hauptdialog
BOOL CALLBACK DialogProcCol(HWND, UINT, WPARAM, LPARAM);	//Farbverwaltung
BOOL CALLBACK DialogProcOK(HWND, UINT, WPARAM, LPARAM);		//OK-Button-Dlg (Infobox...)

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, PSTR szCmdLine, int iCmdShow)
{
	const char name[]="Mandelbrot Calculator";

	MSG msg;
	WNDCLASS wc;

	wc.cbClsExtra=0;
	wc.cbWndExtra=0;
	wc.hbrBackground=(HBRUSH)GetStockObject(WHITE_BRUSH);
	wc.hCursor=LoadCursor(NULL, IDC_ARROW);
	wc.hIcon=LoadIcon(hInstance, MAKEINTRESOURCE(ID_ICON));
	wc.hInstance=hInstance;
	wc.lpfnWndProc=WndProc;
	wc.lpszClassName=name;
	wc.lpszMenuName=NULL;
	wc.style=CS_HREDRAW | CS_VREDRAW;

	RegisterClass(&wc);

	hWnd=CreateWindow(name, name, WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,NULL, NULL, hInstance, NULL);

	ShowWindow(hWnd, SW_HIDE);
	UpdateWindow(hWnd);

	DialogBox(hInstance, "IDD_MAINDLG", hWnd, DialogProc);
	
	while(GetMessage(&msg,NULL,0,0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
	return msg.wParam;
};

LRESULT CALLBACK WndProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	static HINSTANCE hInstance;

	if (message==WM_DESTROY)
	{
		PostQuitMessage(0);
		return 0;
	}

	return DefWindowProc(hWnd, message, wParam, lParam);
};

BOOL CALLBACK DialogProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	HWND hPaint = GetDlgItem(hDlg, ID_PAINT);
    
	if(message==WM_INITDIALOG) 
	{
        FillDialog(hDlg);		//Dialog mit Werten fuellen
		Update(hDlg, hPaint);	//Menge berechnen und darstellen
		return 0;
	}

	if(message==WM_COMMAND)
	{
		switch(LOWORD(wParam))
		{
		case(ID_ZOOM_IN):
			Zoom(zIN, hDlg, hPaint);
			break;
		case(ID_ZOOM_OUT):
			Zoom(zOUT, hDlg, hPaint);
			break;
		case(ID_COLOURS):
			DialogBox(hInstance, "IDD_FARBEN", hDlg, DialogProcCol);
			Update(hDlg, hPaint);
			break;
		case(ID_JULIA):		//Julia-Radiobutton
			Menge.setIsMandel(false);
			Menge.m_JuliaMittelpunkt.set(0,0);
			Menge.setAufloesung(2);
			FillDialog(hDlg);		
		case(ID_MANDEL):	//Mandelbrot-Radiobutton
		case(IDOK):
		case(ID_UPDATE):
			Update(hDlg, hPaint);
			break;
		case(IDCANCEL):		//Dialog schliessen
			EndDialog(hDlg, 0);
			SendMessage(hWnd, WM_CLOSE, 0, 0);
			break;
		case(ID_ABOUT):
			DialogBox(hInstance, "IDD_INFO", hDlg, DialogProcOK);
			break;
		}
	}

	if(message == WM_MOUSEMOVE)
	{
		GetCursorPos(&pt);
		ScreenToClient(hPaint, &pt);

		if( (pt.x > 0) &&		//wenn Mauszeiger innerhalb Darstellung
			(pt.x < XSIZE) &&
			(pt.y > 0) &&
			(pt.y < YSIZE))
		{
			//Clientkoordinaten in Mandelbrotkoordinaten umrechnen:
			double TempRe = Menge.ScreenToMB(Menge.m_Mittelpunkt.getRe(), pt.x, X);
			double TempIm = Menge.ScreenToMB(Menge.m_Mittelpunkt.getIm(), pt.y, Y);
			
			cmplx TempC;	TempC.Re = TempRe;
							TempC.Im = TempIm;
		
			//Anzeigen der Koordinaten und Iterationen:
			sprintf(schrift, "%.17f", TempRe);
			SetDlgItemText(hDlg, ID_MOUSE_RE, schrift);
			sprintf(schrift, "%.17f", TempIm);
			SetDlgItemText(hDlg, ID_MOUSE_IM, schrift);
			SetDlgItemInt(hDlg, ID_MOUSE_IT, Menge.Iterationsschleife(TempC, TempC), true);
		}
	}

	if(message == WM_LBUTTONDOWN)
	{
		GetDlgValues(hDlg);

		GetCursorPos(&pt);
		ScreenToClient(hPaint, &pt);
        
		if( (pt.x > 0) &&
			(pt.x < XSIZE) &&
			(pt.y > 0) &&
			(pt.y < YSIZE))
		{
			if(Menge.isMandel() == true)
			{
				//wenn Mandelbrotmenge angezeigt wird, Mandelbrotmengenmittelpunkt setzen...
				Menge.m_Mittelpunkt.setRe( Menge.ScreenToMB(Menge.m_Mittelpunkt.getRe(), pt.x, X) );
				Menge.m_Mittelpunkt.setIm( Menge.ScreenToMB(Menge.m_Mittelpunkt.getIm(), pt.y, Y) );
            }
			else
			{	//...sonst Juliamittelpunkt setzen
				Menge.m_JuliaMittelpunkt.setRe( Menge.ScreenToMB(Menge.m_JuliaMittelpunkt.getRe(), pt.x, X) );
				Menge.m_JuliaMittelpunkt.setIm( -Menge.ScreenToMB(Menge.m_JuliaMittelpunkt.getIm(), pt.y, Y) );
			}
            
			FillDialog(hDlg);
			Zoom(zIN, hDlg, hPaint);
		}
	}

	if(message == WM_RBUTTONDOWN)
	{
		Zoom(zOUT, hDlg, hPaint);
	}

	if(message==WM_PAINT)
	{
		DrawFraktal(hPaint);
	}

	return FALSE;
}


BOOL CALLBACK DialogProcCol(HWND hDlgCol, UINT message, WPARAM wParam, LPARAM lParam)
{
	if(message==WM_INITDIALOG) 
	{
		SetDlgItemInt(hDlgCol, ID_MB_R, Menge.m_Col_Menge.getRed(), true);
		SetDlgItemInt(hDlgCol, ID_MB_G, Menge.m_Col_Menge.getGreen(), true);
		SetDlgItemInt(hDlgCol, ID_MB_B, Menge.m_Col_Menge.getBlue(), true);

		SetDlgItemInt(hDlgCol, ID_1_R, Menge.m_Col_Lower.getRed(), true);
		SetDlgItemInt(hDlgCol, ID_1_G, Menge.m_Col_Lower.getGreen(), true);
		SetDlgItemInt(hDlgCol, ID_1_B, Menge.m_Col_Lower.getBlue(), true);

		SetDlgItemInt(hDlgCol, ID_2_R, Menge.m_Col_Upper.getRed(), true);
		SetDlgItemInt(hDlgCol, ID_2_G, Menge.m_Col_Upper.getGreen(), true);
		SetDlgItemInt(hDlgCol, ID_2_B, Menge.m_Col_Upper.getBlue(), true);

		return 0;
	}

	if(message==WM_COMMAND)
	{
		switch(LOWORD(wParam))
		{
		case(IDOK):
			Menge.m_Col_Menge.setFarbe(	GetDlgItemInt(hDlgCol, ID_MB_R, NULL, true),
										GetDlgItemInt(hDlgCol, ID_MB_G, NULL, true),
										GetDlgItemInt(hDlgCol, ID_MB_B, NULL, true));

			Menge.m_Col_Lower.setFarbe(	GetDlgItemInt(hDlgCol, ID_1_R, NULL, true),
										GetDlgItemInt(hDlgCol, ID_1_G, NULL, true),
										GetDlgItemInt(hDlgCol, ID_1_B, NULL, true));

			Menge.m_Col_Upper.setFarbe(	GetDlgItemInt(hDlgCol, ID_2_R, NULL, true),
										GetDlgItemInt(hDlgCol, ID_2_G, NULL, true),
										GetDlgItemInt(hDlgCol, ID_2_B, NULL, true));
		case(IDCANCEL):
			EndDialog(hDlgCol, 0);
			break;
		}
	}

	return FALSE;
}


BOOL CALLBACK DialogProcOK(HWND hDlgOK, UINT message, WPARAM wParam, LPARAM lParam)
{
	if(message==WM_COMMAND)
	{
		switch(LOWORD(wParam))
		{
		case(IDOK):
			EndDialog(hDlgOK, 0);
			break;
		}
	}

	return FALSE;
}

